///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////
// Standard wxWidget includes
#include "wx/wx.h"
#include "wx/filename.h"
#include "CEGUI.h"
#include "CEGUIXMLParser.h"
#include "CEGUIImageset.h"
#include "renderers/OpenGLGUIRenderer/opengltexture.h"

#include "CEImagesetEditor.h"
// More specific wxWidgets includes
#include "wx/docview.h"
#include "EditorDocument.h"
#include "EditorView.h"
#include "PropertiesPanel.h"
#include "StringHelper.h"
#include "ImagesetHandler.h"

using namespace CEGUI;


IMPLEMENT_DYNAMIC_CLASS(EditorDocument, wxDocument)

/*************************************************************************
Definition of constant data for Imageset (and sub-classes)
*************************************************************************/
// Declared in Imageset::xmlHandler
const wxPoint  EditorDocument::defaultNativeRes( wxPoint(640, 480) );

/////////////////////////////////////////////////////////////////////////////
// EditorDocument construction/destruction
EditorDocument::EditorDocument()
{
    m_panelProperties = 0;
    reset();
}

/** Handles the wxID_OPEN menu event.
* The system will check whether modifications were made and give the option to save the
* current layout first (if any).
*/
bool EditorDocument::OnOpenDocument(const wxString& filename)
{
    if (filename.empty())
        return false;


    // create handler object
    ImagesetHandler handler(this);

    // do parse (which uses handler to create actual data)
    try
    {
        System::getSingleton().getXMLParser()->parseXMLFile(handler, StringHelper::ToCEGUIString(filename), "Imageset.xsd","");
    }
    catch(...)
    {
        //unload();

        Logger::getSingleton().logEvent("Imageset::load - loading of Imageset from file '" + StringHelper::ToCEGUIString(filename) +"' failed.", Errors);
        throw;
    }


    // Since we have overwritten this method, we must notify the views ourselves!
    UpdateAllViews();

    SetFilename(filename, true);
    SetTitle(filename);


    return true;

}

void findandreplace( String & source, const String& find, const String& replace )
{
    size_t j;
    for (;(j = source.find( find )) != std::string::npos;)
    {
        source.replace( j, find.length(), replace );
    }
}

//-----------------------------------------------------------------------
bool EditorDocument::OnSaveDocument (const wxString& filename)
{

    Renderer * guiRenderer = System::getSingleton().getRenderer();
    Texture* textureImageset = guiRenderer->createTexture(StringHelper::ToCEGUIString(m_filenameImageset), "");

    const String imagesetName (StringHelper::ToCEGUIString(m_nameImageset));
    const String imagesetFileName (StringHelper::ToCEGUIString(m_filenameImageset));

    //remove from previous saves
    if (ImagesetManager::getSingleton().isImagesetPresent (imagesetName))
        ImagesetManager::getSingleton().destroyImageset(imagesetName);

    Imageset* newImageset = ImagesetManager::getSingleton().createImagesetFromImageFile(
        imagesetName, 
        imagesetFileName);

    newImageset->setAutoScalingEnabled(m_bAutoScaled);
    newImageset->setNativeResolution(Size(m_ptNativeRes.x, m_ptNativeRes.y));


    if (newImageset->isImageDefined ("full_image"))
        newImageset->undefineImage("full_image");

    // CEGUI::WindowManager& wMgr = CEGUI::WindowManager::getSingleton();
    // CEGUI::Window* w = (CEGUI::Window*) wMgr.getWindow(StringHelper::ToCEGUIString (m_nameImageset));
    // w->destroy();

    // iterate over all the elements in the class
    mapNamedRegion::iterator it;
    for( it = m_mapSetRectangles.begin(); it != m_mapSetRectangles.end(); ++it )
    {   
        const wxString imageName = it->first;
        const String ImageNameSave = StringHelper::ToCEGUIString(imageName);
        if (!newImageset->isImageDefined(ImageNameSave))
        {
            const wxRect r = it->second;
            const Rect newDim(((r.GetLeft()      > 0) ? r.GetLeft()     : 0), 
                ((r.GetTop()       > 0) ? r.GetTop()      : 0), 
                ((r.GetRight()+1   > 0) ? r.GetRight()+1  : 0), 
                ((r.GetBottom()+1  > 0) ? r.GetBottom()+1 : 0));
            const Point p (0.0f, 0.0f);
            newImageset->defineImage(ImageNameSave, newDim, p);
        }
        //wxString key = it->first, value = it->second;
    }

    {
        std::ofstream fileSave;

        fileSave.open ((const char*)wxConvLibc.cWX2MB(filename), std::ios::out) ;
        if (!fileSave.is_open ()) 
        {
            wxLogError (wxT("Error: could not open file for saving. Is it read-only?")) ;
            return false;
        }

        //newImageset->writeXMLToStream(fileSave);
        ImagesetManager::getSingleton().writeImagesetToStream(imagesetName, fileSave);
        fileSave.close () ; 
    }
        // Hack cegui bug in 0.4.1 version
    {
        std::ifstream file;
        file.open((const char*)wxConvLibc.cWX2MB(filename));
        if (!file.is_open ()) 
        {
            wxLogError (wxT("Error: could not open file for saving. Is it read-only?")) ;
            return false;
        }
        std::stringstream input;
        char buffer[2] = {0};
        while (file)
        {
            file.read(buffer, 1);
            input << buffer;
        }

        String str = input.str();
        String imagefileStr = "Imagefile=\"" + imagesetFileName + "\"";
        findandreplace(str, String("Filename=\"\""), imagefileStr);
        findandreplace(str, String("True"), String("true"));
        {
            std::ofstream fileSave;

            fileSave.open ((const char*)wxConvLibc.cWX2MB(filename), std::ios::out) ;
            if (!fileSave.is_open ()) 
            {
                wxLogError (wxT("Error: could not open file for saving. Is it read-only?")) ;
                return false;
            }
            fileSave <<  str.c_str();
            fileSave.close (); 
        }
    }
    
    // Tell the document it's "clean" again
    Modify(false);
    
    return true;
}

//-----------------------------------------------------------------------
bool EditorDocument::OnCloseDocument ()
{
    //HandleDelAllRegion();

    HandleChangeImagesetName("");
    HandleChangeImageFile("");
    HandleChangeNativeRes(defaultNativeRes);
    HandleChangeAutoScaled(false);
    HandleDelAllRegion();

    //wxString filename("");
    //SetFilename(filename, true);
    //SetTitle(filename);

    return true;
}

//-----------------------------------------------------------------------
bool EditorDocument::OnNewDocument () 
{
    wxDocument::OnNewDocument();

    HandleChangeImageFile("");
    HandleChangeImagesetName("");
    HandleChangeNativeRes(defaultNativeRes);
    HandleChangeAutoScaled(m_bAutoScaled);

    // Since we have overwritten this method, we must notify the views ourselves!
    UpdateAllViews();

    // Tell the document it's "clean" again
    Modify(true);

    return true;
}

//-----------------------------------------------------------------------
void EditorDocument::reset () 
{
    // Document data
    //HandleChangeImageFile("");
    //HandleChangeImagesetName("");
    //HandleChangeNativeRes(wxPoint(0, 0));
    //HandleChangeAutoScaled(false);
    //HandleDelAllRegion();

}

// possible actions on the document 
void EditorDocument::HandleChangeImageFile(const wxString& newfilename)
{
    m_filenameImageset = newfilename;
    static_cast<EditorView*>(GetFirstView())->OnChangeImageFile(newfilename);
}

void EditorDocument::HandleChangeImagesetName(const wxString& newname)
{
    m_nameImageset = newname;
    static_cast<EditorView*>(GetFirstView())->OnChangeImagesetName(newname);
}

void EditorDocument::HandleChangeNativeRes(const wxPoint& newnativeres)
{
    m_ptNativeRes = newnativeres;
    static_cast<EditorView*>(GetFirstView())->OnChangeNativeRes(newnativeres);
}

void EditorDocument::HandleChangeAutoScaled(const bool newAutoScaled)
{
    m_bAutoScaled = newAutoScaled;
    static_cast<EditorView*>(GetFirstView())->OnChangeAutoScaled(newAutoScaled);
}

void EditorDocument::HandleAddRegion(const wxString &name, const wxRect &dim)
{ 
    m_mapSetRectangles[name] = dim;
    static_cast<EditorView*>(GetFirstView())->OnAddRegion(name, dim);
}

void EditorDocument::HandleDelRegion(const wxString &name)
{
    m_mapSetRectangles.erase(name);
}

void EditorDocument::HandleDelAllRegion()
{
    m_mapSetRectangles.clear();
    static_cast<EditorView*>(GetFirstView())->OnDelAllRegion();
}

void EditorDocument::HandleMoveScaleRegion(const wxString &name, const wxRect &dim)
{
    m_mapSetRectangles[name] = dim;
}

void EditorDocument::HandleRenameRegion(const wxString &currentname, const wxString &newname)
{
    wxRect dim = m_mapSetRectangles[currentname];
    m_mapSetRectangles.erase(currentname);
    m_mapSetRectangles[newname] = dim;
}

//-----------------------------------------------------------------------
void EditorDocument::Modify(bool modify)
{
    wxDocument::Modify(modify);
    UpdateAllViews();
}
