/************************************************************************
	filename: 	CEGUIMoveScaleBox.h
	created:	13/4/2004
	author:		Paul D Turner

	purpose:	Interface to base class for FrameWindow
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#ifndef _CEGUIMoveScaleBox_h_
#define _CEGUIMoveScaleBox_h_

#include "CEGUIBase.h"
#include "CEGUIWindow.h"
#include "elements/CEGUIMoveScaleBoxProperties.h"


#if defined(_MSC_VER)
#	pragma warning(push)
#	pragma warning(disable : 4251)
#endif


// Start of CEGUI namespace section
namespace CEGUI
{
/*!
\brief
	Abstract base class for a movable, sizable, window with a title-bar and a frame.
*/
class CEGUIEXPORT MoveScaleBox : public Window
{
public:
	static const String EventNamespace;				//!< Namespace for global events


	/*************************************************************************
		Constants
	*************************************************************************/
	// additional event names for this window
	//static const String EventRollupToggled;		//!< Fired when the rollup (shade) state of the window changes
	//static const String EventCloseClicked;		//!< Fired when the close button for the window is clicked.

	// other bits
	static const float	DefaultScalingBorderSize;	//!< Default size for the sizing border (in pixels)


	/*!
	\brief
		Enumeration that defines the set of possible locations for the mouse on a frame windows sizing border.
	*/
	enum ScalingLocation {
		ScalingNone,			//!< Position is not a sizing location.
		ScalingTopLeft,		//!< Position will size from the top-left.
		ScalingTopRight,		//!< Position will size from the top-right.
		ScalingBottomLeft,	//!< Position will size from the bottom left.
		ScalingBottomRight,	//!< Position will size from the bottom right.
		ScalingTop,			//!< Position will size from the top.
		ScalingLeft,			//!< Position will size from the left.
		ScalingBottom,		//!< Position will size from the bottom.
		ScalingRight         //!< Position will size from the right.
	};

	/*!
	\brief
		Initialises the Window based object ready for use.

	\note
		This must be called for every window created.  Normally this is handled automatically by the WindowFactory for each Window type.

	\return
		Nothing
	*/
	virtual void	initialise(void);


	/*!
	\brief
		Return whether this window is movable.  Note that this requires that the window have an enabled frame and that sizing itself is enabled

	\return
		true if the window can be sized, false if the window can not be sized
	*/
	bool	isMovingEnabled(void) const					{return d_movingEnabled;}

	/*!
	\brief
		Return whether this window is movable.  Note that this requires that the window have an enabled frame and that sizing itself is enabled

	\return
		true if the window can be sized, false if the window can not be sized
	*/
	bool	isBeingMoved(void) const					{return d_beingMoved;}

  /*!
	\brief
		Return whether this window is sizable.  Note that this requires that the window have an enabled frame and that sizing itself is enabled

	\return
		true if the window can be sized, false if the window can not be sized
	*/
	bool	isScalingEnabled(void) const					{return d_scalingEnabled;}

	/*!
	\brief
		Return whether this window is movable.  Note that this requires that the window have an enabled frame and that sizing itself is enabled

	\return
		true if the window can be sized, false if the window can not be sized
	*/
	bool	isBeingScaled(void) const					{return d_beingScaled;}

	/*!
	\brief
		Return the thickness of the sizing border.

	\return
		float value describing the thickness of the sizing border in screen pixels.
	*/
	float	getScalingBorderThickness(void) const		{return d_borderScale;}


	/*!
	\brief
		Enables or disables sizing for this window.

	\param setting
		set to true to enable sizing (also requires frame to be enabled), or false to disable sizing.

	\return
		nothing
	*/
	void	setScalingEnabled(bool setting);

	/*!
	\brief
		Set the size of the sizing border for this window.

	\param pixels
		float value specifying the thickness for the sizing border in screen pixels.

	\return
		Nothing.
	*/
	void	setScalingBorderThickness(float pixels)		{d_borderScale = pixels;}

	/*!
	\brief
		Return whether this MoveScaleBox can be moved by dragging the title bar.

	\return
		true if the Window will move when the user drags the title bar, false if the window will not move.
	*/
	bool	isDragMovingEnabled(void) const		{return d_movingEnabled;}


	/*!
	\brief
		Set whether this MoveScaleBox can be moved by dragging the title bar.

	\param setting
		true if the Window should move when the user drags the title bar, false if the window should not move.

	\return
		Nothing.
	*/
	void	setMovingEnabled(bool setting);

	/*!
	\brief
		Move the window by the pixel offsets specified in \a offset.

		This is intended for internal system use - it is the method by which the title bar moves the frame window.

	\param offset
		Vector2 object containing the offsets to apply (offsets are in screen pixels).

	\return
		Nothing.
	*/
	void	offsetPixelPosition(const Vector2& offset);


    /*!
    \brief
        Return a pointer to the currently set Image to be used for the north-south
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getNSScalingCursorImage() const;

    /*!
    \brief
        Return a pointer to the currently set Image to be used for the east-west
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getEWScalingCursorImage() const;

    /*!
    \brief
        Return a pointer to the currently set Image to be used for the northwest-southeast
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getNWSEScalingCursorImage() const;

    /*!
    \brief
        Return a pointer to the currently set Image to be used for the northeast-southwest
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getNESWScalingCursorImage() const;

    /*!
    \brief
        Return a pointer to the currently set Image to be used for the moving
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getMovingCursorImage() const;

    /*!
    \brief
        Set the Image to be used for the north-south sizing mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setNSScalingCursorImage(const Image* image);

    /*!
    \brief
        Set the Image to be used for the east-west sizing mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setEWScalingCursorImage(const Image* image);

    /*!
    \brief
        Set the Image to be used for the northwest-southeast sizing mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setNWSEScalingCursorImage(const Image* image);

    /*!
    \brief
        Set the Image to be used for the northeast-southwest sizing mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setNESWScalingCursorImage(const Image* image);

    /*!
    \brief
        Set the Image to be used for the moving mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setMovingCursorImage(const Image* image);

    /*!
    \brief
        Set the image to be used for the north-south sizing mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setNSScalingCursorImage(const String& imageset, const String& image);

    /*!
    \brief
        Set the image to be used for the east-west sizing mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setEWScalingCursorImage(const String& imageset, const String& image);

    /*!
    \brief
        Set the image to be used for the northwest-southeast sizing mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setNWSEScalingCursorImage(const String& imageset, const String& image);

    /*!
    \brief
        Set the image to be used for the northeast-southwest sizing mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setNESWScalingCursorImage(const String& imageset, const String& image);

    /*!
    \brief
        Set the image to be used for the moving mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setMovingCursorImage(const String& imageset, const String& image);

    // overridden from Window class
    bool    isHit(const Point& position) const      { return Window::isHit(position); }


	/*************************************************************************
		Construction / Destruction
	*************************************************************************/
	/*!
	\brief
		Constructor for MoveScaleBox objects.
	*/
	MoveScaleBox(const String& name, const String& type);

	/*!
	\brief
		Destructor for FramwWindow objects.
	*/
	virtual ~MoveScaleBox(void);


protected:
	/*************************************************************************
		Implementation Functions
	*************************************************************************/

	/*!
	\brief
		move the window's left edge by 'delta'.  The rest of the window does not move, thus this changes the size of the Window.

	\param delta
		float value that specifies the amount to move the window edge, and in which direction.  Positive values make window smaller.
	*/
	void	moveLeftEdge(float delta);


	/*!
	\brief
		move the window's right edge by 'delta'.  The rest of the window does not move, thus this changes the size of the Window.

	\param delta
		float value that specifies the amount to move the window edge, and in which direction.  Positive values make window larger.
	*/
	void	moveRightEdge(float delta);


	/*!
	\brief
		move the window's top edge by 'delta'.  The rest of the window does not move, thus this changes the size of the Window.

	\param delta
		float value that specifies the amount to move the window edge, and in which direction.  Positive values make window smaller.
	*/
	void	moveTopEdge(float delta);


	/*!
	\brief
		move the window's bottom edge by 'delta'.  The rest of the window does not move, thus this changes the size of the Window.

	\param delta
		float value that specifies the amount to move the window edge, and in which direction.  Positive values make window larger.
	*/
	void	moveBottomEdge(float delta);


	/*!
	\brief
		check local pixel co-ordinate point 'pt' and return one of the
		SizingLocation enumerated values depending where the point falls on
		the sizing border.

	\param pt
		Point object describing, in pixels, the window relative offset to check.

	\return
		One of the ScalingLocation enumerated values that describe which part of
		the sizing border that \a pt corresponded to, if any.
	*/
	ScalingLocation	getScalingBorderAtPoint(const Point& pt) const;


	/*!
	\brief
		return true if given ScalingLocation is on left edge.

	\param loc
		SizingLocation value to be checked.

	\return
		true if \a loc is on the left edge.  false if \a loc is not on the left edge.
	*/
	bool	isLeftScalingLocation(ScalingLocation loc) const			{return ((loc == ScalingLeft) || (loc == ScalingTopLeft) || (loc == ScalingBottomLeft));}


	/*!
	\brief
		return true if given ScalingLocation is on right edge.

	\param loc
		ScalingLocation value to be checked.

	\return
		true if \a loc is on the right edge.  false if \a loc is not on the right edge.
	*/
	bool	isRightScalingLocation(ScalingLocation loc) const			{return ((loc == ScalingRight) || (loc == ScalingTopRight) || (loc == ScalingBottomRight));}


	/*!
	\brief
		return true if given ScalingLocation is on top edge.

	\param loc
		ScalingLocation value to be checked.

	\return
		true if \a loc is on the top edge.  false if \a loc is not on the top edge.
	*/
	bool	isTopScalingLocation(ScalingLocation loc) const			{return ((loc == ScalingTop) || (loc == ScalingTopLeft) || (loc == ScalingTopRight));}


	/*!
	\brief
		return true if given ScalingLocation is on bottom edge.

	\param loc
		ScalingLocation value to be checked.

	\return
		true if \a loc is on the bottom edge.  false if \a loc is not on the bottom edge.
	*/
	bool	isBottomScalingLocation(ScalingLocation loc) const		{return ((loc == ScalingBottom) || (loc == ScalingBottomLeft) || (loc == ScalingBottomRight));}


	/*!
	\brief
		Add frame window specific events
	*/
	void	addMoveScaleBoxEvents(void);

	/*!
	\brief
		Set the appropriate mouse cursor for the given window-relative pixel point.
	*/
	void	setCursorForPoint(const Point& pt) const;


	/*!
	\brief
		Return a Rect that describes, in window relative pixel co-ordinates, the outer edge of the sizing area for this window.
	*/
	virtual	Rect	getScalingRect(void) const		{return Rect(0, 0, getAbsoluteWidth(), getAbsoluteHeight());}


	/*!
	\brief
		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.

	\param class_name
		The class name that is to be checked.

	\return
		true if this window was inherited from \a class_name. false if not.
	*/
	virtual bool	testClassName_impl(const String& class_name) const
	{
		if (class_name==(const utf8*)"MoveScaleBox")	return true;
		return Window::testClassName_impl(class_name);
	}


	/*************************************************************************
		New events for Frame Windows
	*************************************************************************/
	/*!
	\brief
		Event generated internally whenever the roll-up / shade state of the window
		changes.
	*/
	//virtual void	onRollupToggled(WindowEventArgs& e);


	/*!
	\brief
		Event generated internally whenever the close button is clicked.
	*/
	//virtual void	onCloseClicked(WindowEventArgs& e);


	/*************************************************************************
		Overridden event handlers
	*************************************************************************/
	virtual void	onMouseMove(MouseEventArgs& e);
	virtual void	onMouseButtonDown(MouseEventArgs& e);
	virtual void	onMouseButtonUp(MouseEventArgs& e);
	virtual void	onCaptureLost(WindowEventArgs& e);
	virtual void	onActivated(ActivationEventArgs& e);
	virtual void	onDeactivated(ActivationEventArgs& e);


	/*************************************************************************
		Implementation Data
	*************************************************************************/

	// drag-sizing data
	bool	d_movingEnabled;	//!< true if moving is enabled for this window.
  bool  d_beingMoved;      //!< true if window is being moved.
	bool	d_scalingEnabled;	//!< true if scaling is enabled for this window.
	bool	d_beingScaled;		//!< true if window is being sized.
	float	d_borderScale;		//!< thickness of the sizing border around this window
	Point	d_dragPoint;	  	//!< point window is being dragged at.

	// images for cursor when on sizing border
	const Image*	d_nsScalingCursor;		//!< North/South sizing cursor image.
	const Image*	d_ewScalingCursor;		//!< East/West sizing cursor image.
	const Image*	d_nwseScalingCursor;		//!< North-West/South-East cursor image.
	const Image*	d_neswScalingCursor;		//!< North-East/South-West cursor image.
	const Image*	d_MovingCursor;		//!< North-East/South-West cursor image.


private:
	/*************************************************************************
		Static Properties for this class
	*************************************************************************/
	static MoveScaleBoxProperties::ScalingEnabled			      d_scalingEnabledProperty;
	static MoveScaleBoxProperties::MovingEnabled			      d_movingEnabledProperty;
	static MoveScaleBoxProperties::ScalingBorderThickness 	d_scalingBorderThicknessProperty;
  static MoveScaleBoxProperties::NSScalingCursorImage   	d_nsScalingCursorProperty;
  static MoveScaleBoxProperties::EWScalingCursorImage   	d_ewScalingCursorProperty;
  static MoveScaleBoxProperties::NWSEScalingCursorImage 	d_nwseScalingCursorProperty;
  static MoveScaleBoxProperties::NESWScalingCursorImage 	d_neswScalingCursorProperty;
  static MoveScaleBoxProperties::MovingCursorImage  	  	d_MovingCursorProperty;


	/*************************************************************************
		Private methods
	*************************************************************************/
	void	addMoveScaleBoxProperties(void);
};

} // End of  CEGUI namespace section

#if defined(_MSC_VER)
#	pragma warning(pop)
#endif

#endif	// end of guard _CEGUIMoveScaleBox_h_
