/************************************************************************
	filename: 	CEGUISkematicPane.h
	created:	13/4/2004
	author:		Paul D Turner

	purpose:	Interface to base class for SkematicPane
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#ifndef _CEGUISkematicPane_h_
#define _CEGUISkematicPane_h_

#include "CEGUIBase.h"
#include "CEGUIWindow.h"
#include "elements/CEGUISkematicPaneProperties.h"


#if defined(_MSC_VER)
#	pragma warning(push)
#	pragma warning(disable : 4251)
#endif


// Start of CEGUI namespace section
namespace CEGUI
{
/*!
\brief
	Abstract base class for a movable, sizable, window with a title-bar and a frame.
*/
class CEGUIEXPORT SkematicPane : public Window
{
public:
	static const String EventNamespace;				//!< Namespace for global events


	/*************************************************************************
		Constants
	*************************************************************************/
	// additional event names for this window
	static const String EventSnapToPixelEnableToggled;		//!< Fired when the SnapToPixel state of the window changes
	static const String EventSnapToGridEnableToggled;		//!< Fired when the SnapToGrid state of the window changes.
	static const String EventGridEnableToggled;		//!< Fired when the SnapToGrid state of the window changes.
  static const String EventSketchElementModified;		//!< Fired when the SnapToGrid state of the window changes.

	/*!
	\brief
		Initialises the Window based object ready for use.

	\note
		This must be called for every window created.  Normally this is handled automatically by the WindowFactory for each Window type.

	\return
		Nothing
	*/
	virtual void	initialise(void);


	/*!
	\brief
		Return whether this window is sizable.  Note that this requires that the window have an enabled frame and that sizing itself is enabled

	\return
		true if the window can be sized, false if the window can not be sized
	*/
	bool	isSnapToPixelEnabled(void) const					{return d_snapToPixelEnabled;}


	/*!
	\brief
		Return whether the frame for this window is enabled.

	\return
		true if the frame for this window is enabled, false if the frame for this window is disabled.
	*/
	bool	isSnapToGridEnabled(void) const					{return d_snapToGridEnabled && isGridEnabled();}


	/*!
	\brief
		Return whether the title bar for this window is enabled.

	\return
		true if the window has a title bar and it is enabled, false if the window has no title bar or if the title bar is disabled.
	*/
	bool	isGridEnabled(void) const				{return d_gridEnabled;}

	/*!
	\brief
		Enables or disables sizing for this window.

	\param setting
		set to true to enable sizing (also requires frame to be enabled), or false to disable sizing.

	\return
		nothing
	*/
	void	setSnapToPixelEnabled(bool setting);


	/*!
	\brief
		Enables or disables the frame for this window.

	\param setting
		set to true to enable the frame for this window, or false to disable the frame for this window.

	\return
		Nothing.
	*/
	void	setSnapToGridEnabled(bool setting);

	/*!
	\brief
		Enables or disables the close button for the frame window.

	\param setting
		Set to true to enable the close button (if one is attached), or false to disable the close button.

	\return
		Nothing.
	*/
	void	setGridEnabled(bool setting);



	/*!
	\brief
		Return the zoom value of the sketch.

	\return
		float value describing the zoom value of the sketch.
	*/
	float	getSketchZoom(void) const		{return d_sketchZoom;}

	/*!
	\brief
		Set the zoom value of the sketch.

	\param factor
		float value specifying the zoom value of the sketch.

	\return
		Nothing.
	*/
	void	setSketchZoom(float factor);



	/*!
	\brief
		Return the minimum zoom value of the sketch.

	\return
		float value describing the minimum zoom value of the sketch.
	*/
	float	getSketchZoomMin(void) const		{return d_sketchZoomMin;}

	/*!
	\brief
		Set the minimum zoom value of the sketch.

	\param factor
		float value specifying the minimum zoom value of the sketch.

	\return
		Nothing.
	*/
	void	setSketchZoomMin(float factor)		{d_sketchZoomMin = factor;}


	/*!
	\brief
		Return the maximum zoom value of the sketch.

	\return
		float value describing the maximum zoom value of the sketch.
	*/
	float	getSketchZoomMax(void) const		{return d_sketchZoomMax;}

	/*!
	\brief
		Set the maximum zoom value of the sketch.

	\param factor
		float value specifying the maximum zoom value of the sketch.

	\return
		Nothing.
	*/
	void	setSketchZoomMax(float factor)		{d_sketchZoomMax = factor;}



	/*!
	\brief
		Return the real width value in pixels of the sketch.

	\return
		int value describing the real width value in pixels of the sketch.
	*/
	int	getSketchRealWidth(void) const		{return d_sketchRealWidth;}

	/*!
	\brief
		Set the real width value in pixels of the sketch.

	\param factor
		int value specifying the real width value in pixels of the sketch.

	\return
		Nothing.
	*/
	void	setSketchRealWidth(int factor)		{d_sketchRealWidth = factor;}


	/*!
	\brief
		Return the real height value in pixels of the sketch.

	\return
		int value describing the real height value in pixels of the sketch.
	*/
	int	getSketchRealHeight(void) const		{return d_sketchRealHeight;}

  	/*!
	\brief
		Get the cursor position in pixels relative to the sketch.

	\param factor
    Nothing.

	\return
		Point value specifying the cursor position in pixels relative to the sketch.
	*/
  Point SkematicPane::getSketchCursorPosition();

	/*!
	\brief
		Set the real height value in pixels of the sketch.

	\param factor
		int value specifying the real height value in pixels of the sketch.

	\return
		Nothing.
	*/
	void	setSketchRealHeight(int factor)		{d_sketchRealHeight = factor;}

	/*!
	\brief
		Move the window by the pixel offsets specified in \a offset.

		This is intended for internal system use - it is the method by which the title bar moves the frame window.

	\param offset
		Vector2 object containing the offsets to apply (offsets are in screen pixels).

	\return
		Nothing.
	*/
	void	offsetSketchPixelPosition(const Vector2& offset);


	/*!
	\brief
		Return the current colour used for rendering the caption text

	\return
		colour value that specifies the colour used when rendering the title bar caption text.
	*/
	colour	getSketchBackgroundColour(void) const;


	/*!
	\brief
		Sets the colour to be used for rendering the caption text.

	\param col
		colour value that specifies the colour to be used when rendering the title bar caption text.

	\return
		Nothing.
	*/
	void	setSketchBackgroundColour(colour col);

    /*!
    \brief
        Return a pointer to the currently set Image to be used for the north-south
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    StaticImage* getSketchWidget() const;


    /*!
    \brief
        Return a pointer to the currently set Image to be used for the north-south
        sizing mouse cursor.

    \return
        Pointer to an Image object, or 0 for none.
    */
    const Image* getSketchBackgroundImage() const;

    /*!
    \brief
        Set the Image to be used for the north-south sizing mouse cursor.

    \param image
        Pointer to an Image object, or 0 for none.

    \return
        Nothing.
    */
    void setSketchBackgroundImage(const Image* image);


    /*!
    \brief
        Set the image to be used for the north-south sizing mouse cursor.

    \param imageset
        String holding the name of the Imageset containing the Image to be used.

    \param image
        String holding the name of the Image to be used.

    \return
        Nothing.

    \exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
    */
    void setSketchBackgroundImage(const String& imageset, const String& image);

     // overridden from Window class
    //bool    isSketchHit(const Point& position) const      { return Window::isHit(position) && !d_rolledup; }


    /*!
    \brief
        Return a pointer to the named window.

    \return
        Pointer to a window object, or 0 for none.
    */
    Window* getSketchElement(const String& name) const;

    /*!
    \brief
        add an element to the sketch.

    \param image
        Pointer to a window to add to the sketch.

    \return
        Nothing.
    */
    void addSketchElement(String &name, Rect &area);

	/*************************************************************************
		Construction / Destruction
	*************************************************************************/
	/*!
	\brief
		Constructor for SkematicPane objects.
	*/
	SkematicPane(const String& name, const String& type);

	/*!
	\brief
		Destructor for FramwWindow objects.
	*/
	virtual ~SkematicPane(void);


protected:
	/*************************************************************************
		Implementation Functions
	*************************************************************************/
	/*!
	\brief
		Create a control based upon the Titlebar base class to be used as the title bar for this window.

    \param name
        String object holding the name that must be used when creating the titlebar.

	\return
		Pointer to an object who's class derives from Titlebar
	*/
	virtual StaticImage*	createSketchBackgroundImage(const String& name) const		= 0;


	/*!
	\brief
		Add frame window specific events
	*/
	void	addSkematicPaneEvents(void);


	/*!
	\brief
		Method to respond to close button click events and fire our close event
	*/
	//bool	handleSketchElementModified(const EventArgs& e);

	/*!
	\brief
		Return a Rect that describes, in window relative pixel co-ordinates, the outer edge of the sizing area for this window.
	*/
	virtual	Rect	getSketchRect(void) const		{return Rect(0, 0, getAbsoluteWidth(), getAbsoluteHeight());}


	/*!
	\brief
		Return a Rect that describes, trhe correct area after zoomed and snapped applied.
	*/
	virtual	Rect	convertToZoomedSnappedArea(const Rect& rect) const;


	/*!
	\brief
		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.

	\param class_name
		The class name that is to be checked.

	\return
		true if this window was inherited from \a class_name. false if not.
	*/
	virtual bool	testClassName_impl(const String& class_name) const
	{
		if (class_name==(const utf8*)"SkematicPane")	return true;
		return Window::testClassName_impl(class_name);
	}


	/*************************************************************************
		New events for Frame Windows
	*************************************************************************/
	/*!
	\brief
		Event generated internally whenever the roll-up / shade state of the window
		changes.
	*/
	virtual void	onSnapToPixelEnableToggled(WindowEventArgs& e);


	/*!
	\brief
		Event generated internally whenever the close button is clicked.
	*/
	virtual void	onSnapToGridEnableToggled(WindowEventArgs& e);


	/*!
	\brief
		Event generated internally whenever the close button is clicked.
	*/
	virtual void	onGridEnableToggled(WindowEventArgs& e);

	/*!
	\brief
		Event generated internally whenever a sketch element is moved or resized.
	*/
  bool	onSketchElementModified(const EventArgs& e);

	/*************************************************************************
		Overridden event handlers
	*************************************************************************/
	//virtual void	onMouseMove(MouseEventArgs& e);
	//virtual void	onMouseButtonDown(MouseEventArgs& e);
	//virtual void	onMouseButtonUp(MouseEventArgs& e);
	//virtual void	onCaptureLost(WindowEventArgs& e);
	//virtual void    onTextChanged(WindowEventArgs& e);
	virtual void	onActivated(ActivationEventArgs& e);
	virtual void	onDeactivated(ActivationEventArgs& e);


	/*************************************************************************
		Implementation Data
	*************************************************************************/
	// frame data
	bool	d_snapToPixelEnabled;		//!< true if window frame should be drawn.
	// window roll-up data
	bool	d_snapToGridEnabled;	//!< true if roll-up of window is allowed.
	// drag-sizing data
	bool	d_gridEnabled;	//!< true if sizing is enabled for this window.

  /** Current sketch zoom. float because it's as relative value to original size egal 1.*/
  float   d_sketchZoom;

  /** minimum sketch zoom */
  float   d_sketchZoomMin;

  /** maximum sketch zoom */
  float   d_sketchZoomMax;

  /** the skecth initial (without zoom) width in pixel.*/
  int d_sketchRealWidth;

  /** the skecth initial (without zoom) height in pixel.*/
  int d_sketchRealHeight;

	// composite controls
	StaticImage*	d_sketchBackgroundImage;				//!< points to the title bar widget.


private:
	/*************************************************************************
		Static Properties for this class
	*************************************************************************/
	static SkematicPaneProperties::SnapToPixelEnabled	  		d_snapToPixelEnabledProperty;
	static SkematicPaneProperties::SnapToGridEnabled	    	d_snapToGridEnabledProperty;
	static SkematicPaneProperties::GridEnabled					  	d_gridEnabledProperty;
	static SkematicPaneProperties::SketchBackgroundImage		d_sketchBackgroundImageProperty;
	static SkematicPaneProperties::SketchZoom		            d_sketchZoomProperty;
	static SkematicPaneProperties::SketchZoomMin	          d_sketchZoomMinProperty;
	static SkematicPaneProperties::SketchZoomMax            d_sketchZoomMaxProperty;
	static SkematicPaneProperties::SketchRealWidth           d_sketchRealWidthProperty;
	static SkematicPaneProperties::SketchRealHeight           d_sketchRealHeightProperty;


	/*************************************************************************
		Private methods
	*************************************************************************/
	void	addSkematicPaneProperties(void);
};

} // End of  CEGUI namespace section

#if defined(_MSC_VER)
#	pragma warning(pop)
#endif

#endif	// end of guard _CEGUISkematicPane_h_
