/************************************************************************
	filename: 	CEGUIMoveScaleBox.cpp
	created:	13/4/2004
	author:		Paul D Turner

	purpose:	Implementation of MoveScaleBox base class
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#include "elements/CEGUIMoveScaleBox.h"
#include "CEGUIMouseCursor.h"
#include "CEGUIWindowManager.h"
#include "CEGUIExceptions.h"
#include "CEGUIImagesetManager.h"
#include "CEGUIImageset.h"

// Start of CEGUI namespace section
namespace CEGUI
{
const String MoveScaleBox::EventNamespace("MoveScaleBox");

/*************************************************************************
	Definitions for Properties
*************************************************************************/
MoveScaleBoxProperties::ScalingEnabled			    MoveScaleBox::d_scalingEnabledProperty;
MoveScaleBoxProperties::MovingEnabled				    MoveScaleBox::d_movingEnabledProperty;
MoveScaleBoxProperties::ScalingBorderThickness	MoveScaleBox::d_scalingBorderThicknessProperty;
MoveScaleBoxProperties::NSScalingCursorImage    MoveScaleBox::d_nsScalingCursorProperty;
MoveScaleBoxProperties::EWScalingCursorImage    MoveScaleBox::d_ewScalingCursorProperty;
MoveScaleBoxProperties::NWSEScalingCursorImage  MoveScaleBox::d_nwseScalingCursorProperty;
MoveScaleBoxProperties::NESWScalingCursorImage  MoveScaleBox::d_neswScalingCursorProperty;
MoveScaleBoxProperties::MovingCursorImage       MoveScaleBox::d_MovingCursorProperty;


/*************************************************************************
	Constants
*************************************************************************/
// additional event names for this window
//const String MoveScaleBox::EventRollupToggled( (utf8*)"RollupToggled" );
//const String MoveScaleBox::EventCloseClicked( (utf8*)"CloseClicked" );

// other bits
const float MoveScaleBox::DefaultScalingBorderSize	= 4.0f;


/*************************************************************************
	Constructor
*************************************************************************/
MoveScaleBox::MoveScaleBox(const String& type, const String& name) :
	Window(type, name)
{
	d_movingEnabled		= true;
	d_beingMoved		= false;
	d_scalingEnabled		= true;
	d_beingScaled		= false;

	d_borderScale		= DefaultScalingBorderSize;

	d_nsScalingCursor = d_ewScalingCursor = d_neswScalingCursor = d_nwseScalingCursor = d_MovingCursor = NULL;

	addMoveScaleBoxEvents();
	addMoveScaleBoxProperties();
}


/*************************************************************************
	Destructor
*************************************************************************/
MoveScaleBox::~MoveScaleBox(void)
{
}


/*************************************************************************
	Initialises the Window based object ready for use.
*************************************************************************/
void MoveScaleBox::initialise(void)
{
}


/*************************************************************************
	Enables or disables sizing for this window.
*************************************************************************/
void MoveScaleBox::setScalingEnabled(bool setting)
{
  	if (d_scalingEnabled != setting)
	{
		d_scalingEnabled = setting;

		// stop dragging now if the setting has been disabled.
		if ((!d_scalingEnabled) && d_beingScaled)
		{
			releaseInput();
		}

		// call event handler.
		//WindowEventArgs args(this);
		//onScalingModeChanged(args);
	}
}


/*************************************************************************
	Enables or disables the frame for this window.
*************************************************************************/
void MoveScaleBox::setMovingEnabled(bool setting)
{
  	if (d_movingEnabled != setting)
	{
		d_movingEnabled = setting;

		// stop dragging now if the setting has been disabled.
		if ((!d_movingEnabled) && d_beingMoved)
		{
			releaseInput();
		}

		// call event handler.
		//WindowEventArgs args(this);
		//onMovingModeChanged(args);
	}
}


/*************************************************************************
	Move the window by the pixel offsets specified in 'offset'.
*************************************************************************/
void MoveScaleBox::offsetPixelPosition(const Vector2& offset)
{
    UVector2 uOffset;

    if (getMetricsMode() == Relative)
    {
        Size sz = getParentSize();

        uOffset.d_x = cegui_reldim((sz.d_width != 0) ? offset.d_x / sz.d_width : 0);
        uOffset.d_y = cegui_reldim((sz.d_height != 0) ? offset.d_y / sz.d_height : 0);
    }
    else
    {
        uOffset.d_x = cegui_absdim(PixelAligned(offset.d_x));
        uOffset.d_y = cegui_absdim(PixelAligned(offset.d_y));
    }

    setWindowPosition(d_area.getPosition() + uOffset);
}


/*************************************************************************
	check local pixel co-ordinate point 'pt' and return one of the
	SizingLocation enumerated values depending where the point falls on
	the sizing border.
*************************************************************************/
MoveScaleBox::ScalingLocation MoveScaleBox::getScalingBorderAtPoint(const Point& pt) const
{
	Rect	frame(getScalingRect());

	// we can only size if the frame is enabled and sizing is on
	if (isScalingEnabled())
	{
		// point must be inside the outer edge
		if (frame.isPointInRect(pt))
		{
			// adjust rect to get inner edge
			frame.d_left	+= d_borderScale;
			frame.d_top		+= d_borderScale;
			frame.d_right	-= d_borderScale;
			frame.d_bottom	-= d_borderScale;

			// detect which edges we are on
			bool top	= (pt.d_y < frame.d_top);
			bool bottom = (pt.d_y >= frame.d_bottom);
			bool left	= (pt.d_x < frame.d_left);
			bool right	= (pt.d_x >= frame.d_right);

			// return appropriate 'SizingLocation' value
			if (top && left)
			{
				return ScalingTopLeft;
			}
			else if (top && right)
			{
				return ScalingTopRight;
			}
			else if (bottom && left)
			{
				return ScalingBottomLeft;
			}
			else if (bottom && right)
			{
				return ScalingBottomRight;
			}
			else if (top)
			{
				return ScalingTop;
			}
			else if (bottom)
			{
				return ScalingBottom;
			}
			else if (left)
			{
				return ScalingLeft;
			}
			else if (right)
			{
				return ScalingRight;
			}

		}

	}

	// deafult: None.
	return ScalingNone;
}


/*************************************************************************
	move the window's left edge by 'delta'.  The rest of the window
	does not move, thus this changes the size of the Window.
*************************************************************************/
void MoveScaleBox::moveLeftEdge(float delta)
{
    float orgWidth = getAbsoluteWidth();
    float adjustment;
    float* minDim;
    float* maxDim;
    URect area(d_area);

    // ensure that we only size to the set constraints.
    //
    // NB: We are required to do this here due to our virtually unique sizing nature; the
    // normal system for limiting the window size is unable to supply the information we
    // require for updating our internal state used to manage the dragging, etc.
    float maxWidth(d_maxSize.d_x.asAbsolute(System::getSingleton().getRenderer()->getWidth()));
    float minWidth(d_minSize.d_x.asAbsolute(System::getSingleton().getRenderer()->getWidth()));
    float newWidth = orgWidth - delta;

    if (newWidth > maxWidth)
        delta = orgWidth - maxWidth;
    else if (newWidth < minWidth)
        delta = orgWidth - minWidth;

    // we use the active metrics mode to decide which component of the window edge to modify
    if (getMetricsMode() == Relative)
    {
        float base = getParentWidth();
        adjustment = (base != 0) ? delta / base : 0;
        minDim = &area.d_min.d_x.d_scale;
        maxDim = &area.d_max.d_x.d_scale;
    }
    else
    {
        adjustment = PixelAligned(delta);
        minDim = &area.d_min.d_x.d_offset;
        maxDim = &area.d_max.d_x.d_offset;
    }

    if (d_horzAlign == HA_RIGHT)
    {
        *maxDim -= adjustment;
    }
    else if (d_horzAlign == HA_CENTRE)
    {
        *maxDim -= adjustment * 0.5f;
        *minDim += adjustment * 0.5f;
    }
    else
    {
        *minDim += adjustment;
    }

    setWindowArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_LEFT);
}
/*************************************************************************
	move the window's right edge by 'delta'.  The rest of the window
	does not move, thus this changes the size of the Window.
*************************************************************************/
void MoveScaleBox::moveRightEdge(float delta)
{
    // store this so we can work out how much size actually changed
    float orgWidth = getAbsoluteWidth();
    float adjustment;
    float* minDim;
    float* maxDim;
    URect area(d_area);

    // ensure that we only size to the set constraints.
    //
    // NB: We are required to do this here due to our virtually unique sizing nature; the
    // normal system for limiting the window size is unable to supply the information we
    // require for updating our internal state used to manage the dragging, etc.
    float maxWidth(d_maxSize.d_x.asAbsolute(System::getSingleton().getRenderer()->getWidth()));
    float minWidth(d_minSize.d_x.asAbsolute(System::getSingleton().getRenderer()->getWidth()));
    float newWidth = orgWidth + delta;

    if (newWidth > maxWidth)
        delta = maxWidth - orgWidth;
    else if (newWidth < minWidth)
        delta = minWidth - orgWidth;

    // we use the active metrics mode to decide which component of the window edge to modify
    if (getMetricsMode() == Relative)
    {
        float base = getParentWidth();
        adjustment = (base != 0) ? delta / base : 0;
        minDim = &area.d_min.d_x.d_scale;
        maxDim = &area.d_max.d_x.d_scale;
    }
    else
    {
        adjustment = PixelAligned(delta);
        minDim = &area.d_min.d_x.d_offset;
        maxDim = &area.d_max.d_x.d_offset;
    }

    *maxDim += adjustment;

    if (d_horzAlign == HA_RIGHT)
    {
        *maxDim += adjustment;
        *minDim += adjustment;
    }
    else if (d_horzAlign == HA_CENTRE)
    {
        *maxDim += adjustment * 0.5f;
        *minDim += adjustment * 0.5f;
    }

    setWindowArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_RIGHT);

    // move the dragging point so mouse remains 'attached' to edge of window
    d_dragPoint.d_x += getAbsoluteWidth() - orgWidth;
}

/*************************************************************************
	move the window's top edge by 'delta'.  The rest of the window
	does not move, thus this changes the size of the Window.
*************************************************************************/
void MoveScaleBox::moveTopEdge(float delta)
{
    float orgHeight = getAbsoluteHeight();
    float adjustment;
    float* minDim;
    float* maxDim;
    URect area(d_area);

    // ensure that we only size to the set constraints.
    //
    // NB: We are required to do this here due to our virtually unique sizing nature; the
    // normal system for limiting the window size is unable to supply the information we
    // require for updating our internal state used to manage the dragging, etc.
    float maxHeight(d_maxSize.d_y.asAbsolute(System::getSingleton().getRenderer()->getHeight()));
    float minHeight(d_minSize.d_y.asAbsolute(System::getSingleton().getRenderer()->getHeight()));
    float newHeight = orgHeight - delta;

    if (newHeight > maxHeight)
        delta = orgHeight - maxHeight;
    else if (newHeight < minHeight)
        delta = orgHeight - minHeight;

    // we use the active metrics mode to decide which component of the window edge to modify
    if (getMetricsMode() == Relative)
    {
        float base = getParentHeight();
        adjustment = (base != 0) ? delta / base : 0;
        minDim = &area.d_min.d_y.d_scale;
        maxDim = &area.d_max.d_y.d_scale;
    }
    else
    {
        adjustment = PixelAligned(delta);
        minDim = &area.d_min.d_y.d_offset;
        maxDim = &area.d_max.d_y.d_offset;
    }

    if (d_vertAlign == VA_BOTTOM)
    {
        *maxDim -= adjustment;
    }
    else if (d_vertAlign == VA_CENTRE)
    {
        *maxDim -= adjustment * 0.5f;
        *minDim += adjustment * 0.5f;
    }
    else
    {
        *minDim += adjustment;
    }

    setWindowArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_TOP);
}


/*************************************************************************
	move the window's bottom edge by 'delta'.  The rest of the window
	does not move, thus this changes the size of the Window.
*************************************************************************/
void MoveScaleBox::moveBottomEdge(float delta)
{
    // store this so we can work out how much size actually changed
    float orgHeight = getAbsoluteHeight();
    float adjustment;
    float* minDim;
    float* maxDim;
    URect area(d_area);

    // ensure that we only size to the set constraints.
    //
    // NB: We are required to do this here due to our virtually unique sizing nature; the
    // normal system for limiting the window size is unable to supply the information we
    // require for updating our internal state used to manage the dragging, etc.
    float maxHeight(d_maxSize.d_y.asAbsolute(System::getSingleton().getRenderer()->getHeight()));
    float minHeight(d_minSize.d_y.asAbsolute(System::getSingleton().getRenderer()->getHeight()));
    float newHeight = orgHeight + delta;

    if (newHeight > maxHeight)
        delta = maxHeight - orgHeight;
    else if (newHeight < minHeight)
        delta = minHeight - orgHeight;

    // we use the active metrics mode to decide which component of the window edge to modify
    if (getMetricsMode() == Relative)
    {
        float base = getParentHeight();
        adjustment = (base != 0) ? delta / base : 0;
        minDim = &area.d_min.d_y.d_scale;
        maxDim = &area.d_max.d_y.d_scale;
    }
    else
    {
        adjustment = PixelAligned(delta);
        minDim = &area.d_min.d_y.d_offset;
        maxDim = &area.d_max.d_y.d_offset;
    }

    *maxDim += adjustment;

    if (d_vertAlign == VA_BOTTOM)
    {
        *maxDim += adjustment;
        *minDim += adjustment;
    }
    else if (d_vertAlign == VA_CENTRE)
    {
        *maxDim += adjustment * 0.5f;
        *minDim += adjustment * 0.5f;
    }

    setWindowArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_BOTTOM);

    // move the dragging point so mouse remains 'attached' to edge of window
    d_dragPoint.d_y += getAbsoluteHeight() - orgHeight;
}


/*************************************************************************
	Add frame window specific events
*************************************************************************/
void MoveScaleBox::addMoveScaleBoxEvents(void)
{
	//addEvent(EventRollupToggled);
	//addEvent(EventCloseClicked);
}


/*************************************************************************
	Set the appropriate mouse cursor for the given window-relative pixel
	point.
*************************************************************************/
void MoveScaleBox::setCursorForPoint(const Point& pt) const
{
	switch(getScalingBorderAtPoint(pt))
	{
	case ScalingTop:
	case ScalingBottom:
		MouseCursor::getSingleton().setImage(d_nsScalingCursor);
		break;

	case ScalingLeft:
	case ScalingRight:
		MouseCursor::getSingleton().setImage(d_ewScalingCursor);
		break;

	case ScalingTopLeft:
	case ScalingBottomRight:
		MouseCursor::getSingleton().setImage(d_nwseScalingCursor);
		break;

	case ScalingTopRight:
	case ScalingBottomLeft:
		MouseCursor::getSingleton().setImage(d_neswScalingCursor);
		break;

	default:
		MouseCursor::getSingleton().setImage(d_MovingCursor);
		break;
	}

}


/*************************************************************************
	Handler for mouse move events
*************************************************************************/
void MoveScaleBox::onMouseMove(MouseEventArgs& e)
{
	// default processing (this is now essential as it controls event firing).
	Window::onMouseMove(e);

	// if we are not the window containing the mouse, do NOT change the cursor
	if (System::getSingleton().getWindowContainingMouse() != this)
	{
		return;
	}

	if (isScalingEnabled() || isMovingEnabled())
	{
		Point localMousePos(screenToWindow(e.position));

		if (getMetricsMode() == Relative)
		{
			localMousePos = relativeToAbsolute(localMousePos);
		}

		if (d_beingScaled)
		{
			ScalingLocation dragEdge = getScalingBorderAtPoint(d_dragPoint);

			// calculate sizing deltas...
			float	deltaX = localMousePos.d_x - d_dragPoint.d_x;
			float	deltaY = localMousePos.d_y - d_dragPoint.d_y;

			// size left or right edges
			if (isLeftScalingLocation(dragEdge))
			{
				moveLeftEdge(deltaX);
			}
			else if (isRightScalingLocation(dragEdge))
			{
				moveRightEdge(deltaX);
			}

			// size top or bottom edges
			if (isTopScalingLocation(dragEdge))
			{
				moveTopEdge(deltaY);
			}
			else if (isBottomScalingLocation(dragEdge))
			{
				moveBottomEdge(deltaY);
			}

		}
    else if (d_beingMoved)
		{
	    Vector2 delta(screenToWindow(e.position));

	    if (getMetricsMode() == Relative)
	    {
		    delta = relativeToAbsolute(delta);
	    }

	    // calculate amount that window has been moved
	    delta -= d_dragPoint;

	    // move the window. 
	    offsetPixelPosition(delta);
    }
		else
		{
			setCursorForPoint(localMousePos);
		}

	} 

	// mark event as handled
	e.handled = true;
}


/*************************************************************************
	Handler for mouse button down events
*************************************************************************/
void MoveScaleBox::onMouseButtonDown(MouseEventArgs& e)
{
	// default processing (this is now essential as it controls event firing).
	Window::onMouseButtonDown(e);

	if (e.button == LeftButton)
	{
		if (isScalingEnabled() || isMovingEnabled())
		{
			// get position of mouse as co-ordinates local to this window.
			Point localPos(screenToWindow(e.position));

			if (getMetricsMode() == Relative)
			{
				localPos = relativeToAbsolute(localPos);
			}

			// if the mouse is on the sizing border
			if (getScalingBorderAtPoint(localPos) != ScalingNone)
			{
				// ensure all inputs come to us for now
				if (captureInput())
				{
					// setup the 'dragging' state variables
					d_beingMoved = false;
					d_beingScaled = true;
					d_dragPoint = localPos;
				}

			}
      else
			{
				// ensure all inputs come to us for now
				if (captureInput())
				{
					// setup the 'dragging' state variables
					d_beingMoved = true;
					d_beingScaled = false;
					d_dragPoint = localPos;
				}

			}

		}

		e.handled = true;
	}

}


/*************************************************************************
	Handler for mouse button up events
*************************************************************************/
void MoveScaleBox::onMouseButtonUp(MouseEventArgs& e)
{
	// default processing (this is now essential as it controls event firing).
	Window::onMouseButtonUp(e);

	if (e.button == LeftButton)
	{
    d_beingMoved = false;
		d_beingScaled = false;
		// release our capture on the input data
		releaseInput();
		e.handled = true;
	}

}


/*************************************************************************
	Handler for when mouse capture is lost
*************************************************************************/
void MoveScaleBox::onCaptureLost(WindowEventArgs& e)
{
	// default processing (this is now essential as it controls event firing).
	Window::onCaptureLost(e);

	// reset sizing state
	d_beingScaled = false;
	// reset moving state
	d_beingMoved = false;

	e.handled = true;
}


/*************************************************************************
    Handler for when this Window is activated
*************************************************************************/
void MoveScaleBox::onActivated(ActivationEventArgs& e)
{
	Window::onActivated(e);
	//d_titlebar->requestRedraw();
}


/*************************************************************************
    Handler for when this Window is deactivated
*************************************************************************/
void MoveScaleBox::onDeactivated(ActivationEventArgs& e)
{
	Window::onDeactivated(e);
	//d_titlebar->requestRedraw();
}


/*************************************************************************
	Add properties for this class
*************************************************************************/
void MoveScaleBox::addMoveScaleBoxProperties(void)
{
	addProperty(&d_scalingEnabledProperty);
	addProperty(&d_movingEnabledProperty);
	addProperty(&d_scalingBorderThicknessProperty);
  addProperty(&d_nsScalingCursorProperty);
  addProperty(&d_ewScalingCursorProperty);
  addProperty(&d_nwseScalingCursorProperty);
  addProperty(&d_neswScalingCursorProperty);
  addProperty(&d_MovingCursorProperty);
}


const Image* MoveScaleBox::getNSScalingCursorImage() const
{
    return d_nsScalingCursor;
}

const Image* MoveScaleBox::getEWScalingCursorImage() const
{
    return d_ewScalingCursor;
}

const Image* MoveScaleBox::getNWSEScalingCursorImage() const
{
    return d_nwseScalingCursor;
}

const Image* MoveScaleBox::getNESWScalingCursorImage() const
{
    return d_neswScalingCursor;
}

const Image* MoveScaleBox::getMovingCursorImage() const
{
    return d_MovingCursor;
}


void MoveScaleBox::setNSScalingCursorImage(const Image* image)
{
    d_nsScalingCursor = image;
}

void MoveScaleBox::setEWScalingCursorImage(const Image* image)
{
    d_ewScalingCursor = image;
}

void MoveScaleBox::setNWSEScalingCursorImage(const Image* image)
{
    d_nwseScalingCursor = image;
}

void MoveScaleBox::setNESWScalingCursorImage(const Image* image)
{
    d_neswScalingCursor = image;
}

void MoveScaleBox::setMovingCursorImage(const Image* image)
{
    d_MovingCursor = image;
}

void MoveScaleBox::setNSScalingCursorImage(const String& imageset, const String& image)
{
    d_nsScalingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
}

void MoveScaleBox::setEWScalingCursorImage(const String& imageset, const String& image)
{
    d_ewScalingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
}

void MoveScaleBox::setNWSEScalingCursorImage(const String& imageset, const String& image)
{
    d_nwseScalingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
}

void MoveScaleBox::setNESWScalingCursorImage(const String& imageset, const String& image)
{
    d_neswScalingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
}

void MoveScaleBox::setMovingCursorImage(const String& imageset, const String& image)
{
    d_MovingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
}

} // End of  CEGUI namespace section
