/************************************************************************
	filename: 	CEGUISkematicPane.cpp
	created:	13/4/2004
	author:		Paul D Turner

	purpose:	Implementation of SkematicPane base class
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#include "elements/CEGUISkematicPane.h"
#include "elements/CEGUIStaticImage.h"
#include "elements/CEGUIMoveScaleBox.h"
#include "CEGUIWindowManager.h"
#include "CEGUIExceptions.h"
#include "CEGUIImagesetManager.h"
#include "CEGUIImageset.h"
#include "CEGUIMouseCursor.h"

// Start of CEGUI namespace section
namespace CEGUI
{
const String SkematicPane::EventNamespace("SkematicPane");

/*************************************************************************
	Definitions for Properties
*************************************************************************/
SkematicPaneProperties::SnapToPixelEnabled	  	SkematicPane::d_snapToPixelEnabledProperty;
SkematicPaneProperties::SnapToGridEnabled     	SkematicPane::d_snapToGridEnabledProperty;
SkematicPaneProperties::GridEnabled					  	SkematicPane::d_gridEnabledProperty;
SkematicPaneProperties::SketchBackgroundImage		SkematicPane::d_sketchBackgroundImageProperty;
SkematicPaneProperties::SketchZoom		          SkematicPane::d_sketchZoomProperty;
SkematicPaneProperties::SketchZoomMin	          SkematicPane::d_sketchZoomMinProperty;
SkematicPaneProperties::SketchZoomMax           SkematicPane::d_sketchZoomMaxProperty;
SkematicPaneProperties::SketchRealWidth         SkematicPane::d_sketchRealWidthProperty;
SkematicPaneProperties::SketchRealHeight        SkematicPane::d_sketchRealHeightProperty;


/*************************************************************************
	Constants
*************************************************************************/
// additional event names for this window
const String SkematicPane::EventSnapToPixelEnableToggled( (utf8*)"SnapToPixelEnableToggled" );
const String SkematicPane::EventSnapToGridEnableToggled( (utf8*)"SnapToGridEnableToggled" );
const String SkematicPane::EventGridEnableToggled( (utf8*)"GridEnableToggled" );
const String SkematicPane::EventSketchElementModified( (utf8*)"SketchElementModified" );


/*************************************************************************
	Constructor
*************************************************************************/
SkematicPane::SkematicPane(const String& type, const String& name) :
	Window(type, name)
{
	d_snapToPixelEnabled	= true;
	d_snapToGridEnabled		= false;
	d_gridEnabled				= false;

  d_sketchZoom = 1.0;
  d_sketchZoomMin = 0.25;
  d_sketchZoomMax = 10.0;
  d_sketchRealWidth = 0;
  d_sketchRealHeight = 0;

	d_sketchBackgroundImage = NULL;

	addSkematicPaneEvents();
	addSkematicPaneProperties();
}


/*************************************************************************
	Destructor
*************************************************************************/
SkematicPane::~SkematicPane(void)
{
}


/*************************************************************************
	Initialises the Window based object ready for use.
*************************************************************************/
void SkematicPane::initialise(void)
{
	// create child windows
	d_sketchBackgroundImage		= createSketchBackgroundImage(getName() + "__auto_sketchimage__");

	// add child controls
	if (d_sketchBackgroundImage != NULL)
	{
		//d_titlebar->setDraggingEnabled(d_dragMovable);
		addChildWindow(d_sketchBackgroundImage);
		//d_closeButton->subscribeEvent(PushButton::EventClicked, Event::Subscriber(&CEGUI::SkematicPane::closeClickHandler, this));
	}

	performChildWindowLayout();
}


/*************************************************************************
	Enables or disables sizing for this window.
*************************************************************************/
void SkematicPane::setSnapToPixelEnabled(bool setting)
{
	d_snapToPixelEnabled = setting;
}


/*************************************************************************
	Enables or disables the frame for this window.
*************************************************************************/
void SkematicPane::setSnapToGridEnabled(bool setting)
{
	d_snapToGridEnabled = setting;
	requestRedraw();
}

/*************************************************************************
	Enables or disables the close button for the frame window.
*************************************************************************/
void SkematicPane::setGridEnabled(bool setting)
{
/*    try
    {
        Window* closebtn = WindowManager::getSingleton().getWindow(getName() + "__auto_closebutton__");
        closebtn->setEnabled(setting);
        closebtn->setVisible(setting);
    }
    catch (UnknownObjectException)
    {}*/
}

/*************************************************************************
	set the Set the zoom value of the sketch between min and max.
*************************************************************************/
void SkematicPane::setSketchZoom(float factor)
{
  // extrem down limit verif
  if (factor < d_sketchZoomMin)
      factor = d_sketchZoomMin;

  // extrem up limit verif
  if (factor > d_sketchZoomMax)
      factor = d_sketchZoomMax;

  d_sketchZoom = factor;

  
  float width = d_sketchZoom * d_sketchRealWidth;
  float height = d_sketchZoom * d_sketchRealHeight;
  d_sketchBackgroundImage->setWidth(MetricsMode::Absolute, width);
  d_sketchBackgroundImage->setHeight(MetricsMode::Absolute, height);
}
 
/*************************************************************************
	Move the window by the pixel offsets specified in 'offset'.
*************************************************************************/
Point SkematicPane::getSketchCursorPosition()
{
    // updating statusbar with the mouse position on the imageset ( computing offset and zoom)
    int mousePosX = d_sketchBackgroundImage->getXPosition(MetricsMode::Absolute);
    int mousePosY = d_sketchBackgroundImage->getYPosition(MetricsMode::Absolute);
    Point posMouse = MouseCursor::getSingleton().getPosition();
    float mouseAbsPosX = posMouse.d_x;
    float mouseAbsPosY = posMouse.d_y;
    mousePosX = (mouseAbsPosX - mousePosX ) / d_sketchZoom;
    mousePosY = (mouseAbsPosY - mousePosY ) / d_sketchZoom;
    return Point(mousePosX, mousePosY);
}

/*************************************************************************
	Move the window by the pixel offsets specified in 'offset'.
*************************************************************************/
void SkematicPane::offsetSketchPixelPosition(const Vector2& offset)
{
	
    UVector2 uOffset;

    if (getMetricsMode() == Relative)
    {
        Size sz = getParentSize();

        uOffset.d_x = cegui_reldim((sz.d_width != 0) ? offset.d_x / sz.d_width : 0);
        uOffset.d_y = cegui_reldim((sz.d_height != 0) ? offset.d_y / sz.d_height : 0);
    }
    else
    {
        uOffset.d_x = cegui_absdim(PixelAligned(offset.d_x));
        uOffset.d_y = cegui_absdim(PixelAligned(offset.d_y));
    }

    d_sketchBackgroundImage->setWindowPosition(d_sketchBackgroundImage->getWindowPosition () + uOffset);
}


/*************************************************************************
	Add frame window specific events
*************************************************************************/
void SkematicPane::addSkematicPaneEvents(void)
{
	addEvent(EventSnapToPixelEnableToggled);
	addEvent(EventSnapToGridEnableToggled);
	addEvent(EventGridEnableToggled);
	addEvent(EventSketchElementModified);
}


/*************************************************************************
	Event generated internally whenever the roll-up / shade state of the
	window changes.
*************************************************************************/
void SkematicPane::onSnapToPixelEnableToggled(WindowEventArgs& e)
{
    requestRedraw();

	fireEvent(EventSnapToPixelEnableToggled, e, EventNamespace);
}


/*************************************************************************
	Event generated internally whenever the close button is clicked.
*************************************************************************/
void SkematicPane::onSnapToGridEnableToggled(WindowEventArgs& e)
{
	fireEvent(EventSnapToGridEnableToggled, e, EventNamespace);
}



/*************************************************************************
	Event generated internally whenever the close button is clicked.
*************************************************************************/
void SkematicPane::onGridEnableToggled(WindowEventArgs& e)
{
	fireEvent(EventGridEnableToggled, e, EventNamespace);
}


/*************************************************************************
    Handler for when this Window is activated
*************************************************************************/
void SkematicPane::onActivated(ActivationEventArgs& e)
{
	Window::onActivated(e);
	d_sketchBackgroundImage->requestRedraw();
}


/*************************************************************************
    Handler for when this Window is deactivated
*************************************************************************/
void SkematicPane::onDeactivated(ActivationEventArgs& e)
{
	Window::onDeactivated(e);
	d_sketchBackgroundImage->requestRedraw();
}

/*************************************************************************
	Handler to map close button clicks to FrameWindow 'CloseCliked' events
*************************************************************************/
/*
bool SkematicPane::handleSketchElementModified(const EventArgs& e)
{
  
  WindowEventArgs args(System::getSingleton().getWindowContainingMouse());
	onSketchElementModified(args);

	return true;
}
*/
/*************************************************************************
    Handler for when an element of the sketch is modified
*************************************************************************/
bool SkematicPane::onSketchElementModified(const EventArgs& e)
{
  // can do this because we are sure the event initiator is a MoveScaleBox widget
  const WindowEventArgs& wEventArgs = static_cast<const WindowEventArgs&>(e);
  MoveScaleBox* win = static_cast<MoveScaleBox*>(wEventArgs.window);

  if(d_snapToPixelEnabled && win && win->testClassName("MoveScaleBox") && (win->isBeingMoved() || win->isBeingScaled()))
  {
    Rect areaSketch = d_sketchBackgroundImage->getAbsoluteRect();
    Rect areaElement = win->getAbsoluteRect();
    int zoom = d_sketchZoom * 100;

    // TODO : check up the algorythm, there is some glitch sometime in the pos/size snapped evaluation
    // process position
    int left = areaElement.d_left * 100;
    int top = areaElement.d_top * 100;
    areaElement.d_left -= (left % zoom) / 100;
    areaElement.d_top -= (top % zoom) / 100;

    // 
    int right = areaElement.d_right * 100;
    int bottom = areaElement.d_bottom * 100;
    areaElement.d_right -= (right % zoom) / 100;
    areaElement.d_bottom -= (bottom % zoom) / 100;

    //win->setRect(MetricsMode::Absolute, areaElement);

    // to set the area in relative metricmode
    areaElement.d_left = areaElement.d_left / areaSketch.getWidth();
    areaElement.d_top  = areaElement.d_top / areaSketch.getHeight();
    areaElement.d_right = areaElement.d_right / areaSketch.getWidth();
    areaElement.d_bottom  = areaElement.d_bottom / areaSketch.getHeight();

    win->setRect(MetricsMode::Relative, areaElement);   
    
  }   
  
  return true;
	//fireEvent(EventSketchElementModified, e, EventNamespace);
}
/*************************************************************************
	Add properties for this class
*************************************************************************/
void SkematicPane::addSkematicPaneProperties(void)
{
	addProperty(&d_snapToPixelEnabledProperty);
	addProperty(&d_snapToGridEnabledProperty);
	addProperty(&d_gridEnabledProperty);
	addProperty(&d_sketchBackgroundImageProperty);
	addProperty(&d_sketchZoomProperty);
	addProperty(&d_sketchZoomMinProperty);
	addProperty(&d_sketchZoomMaxProperty);
	addProperty(&d_sketchRealWidthProperty);
	addProperty(&d_sketchRealHeightProperty);
}


/*************************************************************************
	Return the current colour used for rendering the caption text
*************************************************************************/
colour SkematicPane::getSketchBackgroundColour(void) const
{
    ColourRect colours = d_sketchBackgroundImage->getBackgroundColours();
    return colours.d_top_left;
}


/*************************************************************************
	Sets the colour to be used for rendering the caption text.
*************************************************************************/
void SkematicPane::setSketchBackgroundColour(colour col)
{
  ColourRect colours(col, col, col, col );
    d_sketchBackgroundImage->setBackgroundColours(colours);
}

StaticImage* SkematicPane::getSketchWidget() const
{
  return d_sketchBackgroundImage;
}

const Image* SkematicPane::getSketchBackgroundImage() const
{
    return d_sketchBackgroundImage->getBackgroundImage();
}

void SkematicPane::setSketchBackgroundImage(const Image* image)
{
    d_sketchBackgroundImage->setBackgroundImage(image);
    d_sketchRealWidth = image->getWidth();
    d_sketchRealHeight = image->getHeight();
    d_sketchBackgroundImage->setWindowWidth(UDim(0, d_sketchRealWidth * d_sketchZoom));
    d_sketchBackgroundImage->setWindowHeight(UDim(0, d_sketchRealHeight * d_sketchZoom));

}

void SkematicPane::setSketchBackgroundImage(const String& imageset, const String& image)
{
    const Image& imagebackground = ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
    d_sketchBackgroundImage->setBackgroundImage(&imagebackground);
    d_sketchRealWidth = imagebackground.getWidth();
    d_sketchRealHeight = imagebackground.getHeight();
    d_sketchBackgroundImage->setWindowWidth(UDim(0, d_sketchRealWidth * d_sketchZoom));
    d_sketchBackgroundImage->setWindowHeight(UDim(0, d_sketchRealHeight * d_sketchZoom));
}

Window* SkematicPane::getSketchElement(const String& name) const
{
  return d_sketchBackgroundImage->getChild(name);
}

void SkematicPane::addSketchElement(String &name, Rect &area)
{
  MoveScaleBox* sketchElement = static_cast<MoveScaleBox*>(WindowManager::getSingleton().createWindow("Marti/OutlineWindow", name));
  d_sketchBackgroundImage->addChildWindow(sketchElement);
  sketchElement->subscribeEvent(Window::EventMoved, Event::Subscriber(&CEGUI::SkematicPane::onSketchElementModified, this));
  sketchElement->subscribeEvent(Window::EventSized, Event::Subscriber(&CEGUI::SkematicPane::onSketchElementModified, this));
  //sketchElement->subscribeEvent(Window::EventParentSized, Event::Subscriber(&CEGUI::SkematicPane::onSketchElementModified, this));

  //Rect areaWanted(area);
  //Rect areaCorrected(convertToZoomedSnappedArea(areaWanted));

  Rect areaCorrected(area);

  areaCorrected.d_left /= d_sketchRealWidth;
  areaCorrected.d_top /= d_sketchRealHeight;
  areaCorrected.d_right /= d_sketchRealWidth;
  areaCorrected.d_bottom /= d_sketchRealHeight;

  sketchElement->setRect(MetricsMode::Relative, areaCorrected);

}

Rect SkematicPane::convertToZoomedSnappedArea(const Rect& rect) const
{
  Rect tmp(rect);

  tmp.d_left /= d_sketchRealWidth;
  tmp.d_top /= d_sketchRealHeight;
  tmp.d_right /= d_sketchRealWidth;
  tmp.d_bottom /= d_sketchRealHeight;
  
  return tmp;
}

} // End of  CEGUI namespace section
